@@ -9,17 +9,24 @@ Options:
   -h, --help     Print this help and exit.
   -v, --version  Print module version and exit.
   --branch=<b>   Branch or tag [default: HEAD] up to which to check.
-  --sort=<key>   [default: loc]|commits|files.
+  --sort=<key>   [default: loc]|commits|files|hours|months.
   --excl=<f>     Excluded files (default: None).
                  In no-regex mode, may be a comma-separated list.
                  Escape (\,) for a literal comma (may require \\, in shell).
   --incl=<f>     Included files [default: .*]. See `--excl` for format.
   --since=<date>  Date from which to check. Can be absoulte (eg: 1970-01-31)
                   or relative to now (eg: 3.weeks).
+  --cost=<method>  Include time cost in person-months (COCOMO) or
+                   person-hours (based on commit times).
+                   Methods: month(s)|cocomo|hour(s)|commit(s).
+                   May be multiple comma-separated values.
   -n, --no-regex  Assume <f> are comma-separated exact matches
                   rather than regular expressions [default: False].
                   NB: if regex is enabled `,` is equivalent to `|`.
   -s, --silent-progress    Suppress `tqdm` [default: False].
+  --warn-binary   Don't silently skip files which appear to be binary data
+                  [default: False].
+  -e, --show-email  Show author email instead of name [default: False].
   -t, --bytype             Show stats per file extension [default: False].
   -w, --ignore-whitespace  Ignore whitespace when comparing the parent's
                            version and the child's to find where the lines
@@ -27,8 +34,9 @@ Options:
   -M  Detect intra-file line moves and copies [default: False].
   -C  Detect inter-file line moves and copies [default: False].
   --format=<format>        Table format
-      [default: md]|markdown|yaml|yml|json|csv|tsv|tabulate.
+      [default: pipe]|md|markdown|yaml|yml|json|csv|tsv|tabulate.
       May require `git-fame[<format>]`, e.g. `pip install git-fame[yaml]`.
+      Any `tabulate.tabulate_formats` is also accepted.
   --manpath=<path>         Directory in which to install git-fame man pages.
   --log=<lvl>     FATAL|CRITICAL|ERROR|WARN(ING)|[default: INFO]|DEBUG|NOTSET.
 """
@@ -39,8 +47,8 @@ import subprocess
 import re
 import logging
 
-from ._utils import TERM_WIDTH, int_cast_or_len, Max, fext, _str, \
-    check_output, tqdm, TqdmStream, print_unicode
+from ._utils import TERM_WIDTH, int_cast_or_len, fext, _str, \
+    check_output, tqdm, TqdmStream, print_unicode, Str
 from ._version import __version__  # NOQA
 
 __author__ = "Casper da Costa-Luis <casper@caspersci.uk.to>"
@@ -51,21 +59,34 @@ __copyright__ = ' '.join(("Copyright (c)", __date__, __author__, __licence__))
 __license__ = __licence__  # weird foreign language
 
 
-RE_AUTHS = re.compile('^\w+ \d+ \d+ (\d+)\nauthor (.+)$', flags=re.M)
+RE_AUTHS = re.compile(
+    r'^\w+ \d+ \d+ (\d+)\nauthor (.+?)$.*?\ncommitter-time (\d+)',
+    flags=re.M | re.DOTALL)
 # finds all non-escaped commas
 # NB: does not support escaping of escaped character
 RE_CSPILT = re.compile(r'(?<!\\),')
 RE_NCOM_AUTH_EM = re.compile(r'^\s*(\d+)\s+(.*?)\s+<(.*)>\s*$', flags=re.M)
 
 
-def tr_hline(col_widths, hl='-', x='+'):
-  return x + x.join(hl * i for i in col_widths) + x
+def hours(dates, maxCommitDiffInSec=120 * 60, firstCommitAdditionInMinutes=120):
+  """
+  Convert list of commit times (in seconds) to an estimate of hours spent.
+
+  https://github.com/kimmobrunfeldt/git-hours/blob/\
+8aaeee237cb9d9028e7a2592a25ad8468b1f45e4/index.js#L114-L143
+  """
+  dates = sorted(dates)
+  diffInSec = [i - j for (i, j) in zip(dates[1:], dates[:-1])]
+  res = sum(filter(lambda i: i < maxCommitDiffInSec, diffInSec))
+  return (res / 60.0 + firstCommitAdditionInMinutes) / 60.0
 
 
 def tabulate(
-        auth_stats, stats_tot, sort='loc', bytype=False, backend='md'):
+        auth_stats, stats_tot, sort='loc', bytype=False, backend='md',
+        cost=None):
   """
-  backends  : [default: md]|yaml|json|csv|tsv|tabulate
+  backends  : [default: md]|yaml|json|csv|tsv|tabulate|
+    `in tabulate.tabulate_formats`
   """
   log = logging.getLogger(__name__)
   COL_NAMES = ['Author', 'loc', 'coms', 'fils', ' distribution']
@@ -80,21 +101,37 @@ def tabulate(
               100 * s.get('commits', 0) / max(1, stats_tot['commits']),
               100 * len(s.get('files', [])) / max(1, stats_tot['files'])
           ))).replace('/100.0/', '/ 100/')]
-         for (auth, s) in sorted(it_as(),
-         key=lambda k: int_cast_or_len(k[1].get(sort, 0)),
-         reverse=True)]
+         for (auth, s) in it_as()]
+  if cost is None:
+    cost = ''
+  if cost:
+    cost = cost.lower()
+    stats_tot = dict(stats_tot)
+    if any(i in cost for i in ['cocomo', 'month']):
+      COL_NAMES.insert(1, 'mths')
+      tab = [i[:1] + [3.2 * (i[1] / 1e3)**1.05] + i[1:] for i in tab]
+      stats_tot.setdefault('months', '%.1f' % sum(i[1] for i in tab))
+    if any(i in cost for i in ['commit', 'hour']):
+      COL_NAMES.insert(1, 'hrs')
+      tab = [i[:1] + [hours(auth_stats[i[0]]['ctimes'])] + i[1:] for i in tab]
+
+    stats_tot.setdefault('hours', '%.1f' % sum(i[1] for i in tab))
+  # log.debug(auth_stats)
+
+  for i, j in [
+      ("commits", "coms"), ("files", "fils"), ("hours", "hrs"),
+      ("months", "mths")]:
+    sort = sort.replace(i, j)
+  tab.sort(key=lambda i: i[COL_NAMES.index(sort)], reverse=True)
 
   totals = 'Total ' + '\nTotal '.join(
-      "%s: %d" % i for i in sorted(stats_tot.items())) + '\n'
+      "%s: %s" % i for i in sorted(stats_tot.items())) + '\n'
 
   backend = backend.lower()
-  if backend == 'tabulate':
-    from tabulate import tabulate as tabber
-    log.debug("backend:tabulate")
-    return totals + tabber(tab, COL_NAMES, tablefmt='grid', floatfmt='.0f')
-    # from ._utils import tighten
-    # return totals + tighten(tabber(...), max_width=TERM_WIDTH)
-  elif backend in ['yaml', 'yml', 'json', 'csv', 'tsv']:
+  if backend in ("tabulate", "md", "markdown"):
+    backend = "pipe"
+
+  if backend in ['yaml', 'yml', 'json', 'csv', 'tsv']:
     tab = [i[:-1] + [float(pc.strip()) for pc in i[-1].split('/')] for i in tab]
     tab = dict(
         total=stats_tot, data=tab,
@@ -121,64 +158,21 @@ def tabulate(
       return res.getvalue().rstrip()
     else:  # pragma: nocover
       raise RuntimeError("Should be unreachable")
-  elif backend not in ['md', 'markdown']:
-    raise ValueError("Unknown backend:%s" % backend)
-
-  log.debug("backend:md")
-  # TODO: convert below to separate function for testing
-
-  res = ''
-  stats = list(auth_stats.values())
-  # Columns: Author | loc | coms | fils | distribution
-  COL_LENS = [
-      max(6, Max(len(a) for a in auth_stats)),
-      max(3, Max(len(str(i["loc"])) for i in stats)),
-      max(4, Max(len(str(i.get("commits", 0))) for i in stats)),
-      max(4, Max(len(str(len(i.get("files", [])))) for i in stats)),
-      12
-  ]
-
-  COL_LENS[0] = min(TERM_WIDTH - sum(COL_LENS[1:]) - len(COL_LENS) * 3 - 3,
-                    COL_LENS[0])
-
-  COL_NAMES = [
-      "Author" + ' ' * (COL_LENS[0] - 6),
-      ' ' * (COL_LENS[1] - 3) + "loc",
-      ' ' * (COL_LENS[2] - 4) + "coms",
-      ' ' * (COL_LENS[3] - 4) + "fils",
-      " distribution "
-  ]
-
-  tbl_row_fmt = u"| {0:<%ds}| {1:>%dd} | {2:>%dd} | {3:>%dd} |" \
-                u" {4:4.1f}/{5:4.1f}/{6:4.1f} |" % (COL_LENS[0] + 1,
-                                                    COL_LENS[1],
-                                                    COL_LENS[2],
-                                                    COL_LENS[3])
-
-  TR_HLINE = tr_hline([len(i) + 2 for i in COL_NAMES])
-  res += TR_HLINE + '\n'
-  res += ("| {0:s} | {1:s} | {2:s} | {3:s} | {4} |").format(*COL_NAMES) + '\n'
-  res += tr_hline([len(i) + 2 for i in COL_NAMES], '=') + '\n'
-
-  for (auth, stats) in tqdm(
-      sorted(
-          auth_stats.items(),
-          key=lambda k: int_cast_or_len(k[1].get(sort, 0)),
-          reverse=True), leave=False):
-    # print (stats)
-    loc = stats["loc"]
-    commits = stats.get("commits", 0)
-    files = len(stats.get("files", []))
-    if bytype:
-      log.debug("TODO:NotImplemented:--bytype")
-      # TODO: print ([stats.get("files", []) ])
-    res += (tbl_row_fmt.format(
-        auth[:len(COL_NAMES[0]) + 1], loc, commits, files,
-        100 * loc / max(1, stats_tot["loc"]),
-        100 * commits / max(1, stats_tot["commits"]),
-        100 * files / max(1, stats_tot["files"])).replace('100.0', ' 100')) \
-        + '\n'
-  return totals + res + TR_HLINE
+  else:
+    import tabulate as tabber
+    if backend not in tabber.tabulate_formats:
+      raise ValueError("Unknown backend:%s" % backend)
+    log.debug("backend:tabulate:" + backend)
+    COL_LENS = [max(len(Str(i[j])) for i in [COL_NAMES] + tab)
+                for j in range(len(COL_NAMES))]
+    COL_LENS[0] = min(
+        TERM_WIDTH - sum(COL_LENS[1:]) - len(COL_LENS) * 3 - 4,
+        COL_LENS[0])
+    tab = [[i[0][:COL_LENS[0]]] + i[1:] for i in tab]
+    return totals + tabber.tabulate(
+        tab, COL_NAMES, tablefmt=backend, floatfmt='.0f')
+    # from ._utils import tighten
+    # return totals + tighten(tabber(...), max_width=TERM_WIDTH)
 
 
 def run(args):
@@ -187,9 +181,10 @@ def run(args):
 
   log.debug("parsing args")
 
-  if args.sort not in ["loc", "commits", "files"]:
-    log.warn("--sort argument (" + args.sort +
-             ") unrecognised\n" + __doc__)
+  if args.sort not in "loc commits files hours months".split():
+    log.warn("--sort argument (%s) unrecognised\n%s" % (
+        args.sort, __doc__))
+    raise KeyError(args.sort)
 
   if not args.excl:
     args.excl = ""
@@ -249,20 +244,22 @@ def run(args):
     try:
       blame_out = check_output(git_blame_cmd, stderr=subprocess.STDOUT)
     except Exception as e:
-      log.warn(fname + ':' + str(e))
+      getattr(log, "warn" if args.warn_binary else "debug")(fname + ':' + str(e))
       continue
     log.log(logging.NOTSET, blame_out)
-    loc_auths = RE_AUTHS.findall(blame_out)
+    loc_auth_times = RE_AUTHS.findall(blame_out)
 
-    for loc, auth in loc_auths:  # for each chunk
+    for loc, auth, tstamp in loc_auth_times:  # for each chunk
       loc = int(loc)
       auth = _str(auth)
+      tstamp = int(tstamp)
       try:
         auth_stats[auth]["loc"] += loc
       except KeyError:
-        auth_stats[auth] = {"loc": loc, "files": set([fname])}
+        auth_stats[auth] = {"loc": loc, "files": set([fname]), "ctimes": []}
       else:
         auth_stats[auth]["files"].add(fname)
+        auth_stats[auth]["ctimes"].append(tstamp)
 
       if args.bytype:
         fext_key = ("." + fext(fname)) if fext(fname) else "._None_ext"
@@ -278,13 +275,25 @@ def run(args):
   for stats in auth_stats.values():
     stats.setdefault("commits", 0)
   log.debug(RE_NCOM_AUTH_EM.findall(auth_commits.strip()))
-  for (ncom, auth, _) in RE_NCOM_AUTH_EM.findall(auth_commits.strip()):
+  auth2em = {}
+  for (ncom, auth, em) in RE_NCOM_AUTH_EM.findall(auth_commits.strip()):
+    auth = _str(auth)
+    auth2em[auth] = em  # TODO: count most used email?
     try:
-      auth_stats[_str(auth)]["commits"] += int(ncom)
+      auth_stats[auth]["commits"] += int(ncom)
     except KeyError:
-      auth_stats[_str(auth)] = {"loc": 0,
-                                "files": set([]),
-                                "commits": int(ncom)}
+      auth_stats[auth] = {"loc": 0,
+                          "files": set([]),
+                          "commits": int(ncom),
+                          "ctimes": []}
+  if args.show_email:
+    # replace author name with email
+    log.debug(auth2em)
+    old = auth_stats
+    auth_stats = {}
+    for auth, stats in getattr(old, 'iteritems', old.items)():
+      auth_stats[auth2em[auth]] = stats
+    del old
 
   stats_tot = dict((k, 0) for stats in auth_stats.values() for k in stats)
   log.debug(stats_tot)
@@ -301,7 +310,8 @@ def run(args):
   # log.debug(extns)
 
   print_unicode(tabulate(
-      auth_stats, stats_tot, args.sort, args.bytype, args.format))
+      auth_stats, stats_tot,
+      args.sort, args.bytype, args.format, args.cost))
 
 
 def main(args=None):
@@ -311,7 +321,8 @@ def main(args=None):
                 version=__version__).parse_args(args=args)
   logging.basicConfig(
       level=getattr(logging, args.log, logging.INFO),
-      stream=TqdmStream)
+      stream=TqdmStream,
+      format="%(levelname)s:gitfame.%(funcName)s:%(lineno)d:%(message)s")
   log = logging.getLogger(__name__)
 
   log.debug(args)
