@@ -18,47 +18,96 @@ auth_stats = {
     u'Not Committed Yet': {'files': set([
         'gitfame/_gitfame.py', 'gitfame/_utils.py', 'Makefile', 'MANIFEST.in'
     ]),
-        'loc': 75, 'commits': 0},
+        'loc': 75, 'ctimes': [], 'commits': 0},
     u'Casper da Costa-Luis': {'files': set([
         'gitfame/_utils.py', 'gitfame/__main__.py', 'setup.cfg',
         'gitfame/_gitfame.py', 'gitfame/__init__.py',
         'git-fame_completion.bash', 'Makefile', 'MANIFEST.in', '.gitignore',
-        'setup.py']), 'loc': 538, 'commits': 35}
+        'setup.py']), 'loc': 538, 'ctimes': [
+        1510942009, 1517426360, 1532103452, 1543323944, 1548030670, 1459558286,
+        1510942009, 1459559144, 1481150373, 1510942009, 1548030670, 1517178199,
+        1481150379, 1517426360, 1548030670, 1459625059, 1510942009, 1517426360,
+        1481150373, 1517337751, 1517426360, 1510942009, 1548030670, 1459099074,
+        1459598664, 1517337751, 1517176447, 1552697404, 1546630326, 1543326881,
+        1459558286, 1481150373, 1510930168, 1459598664, 1517596988],
+        'commits': 35}
 }
 stats_tot = {'files': 14, 'loc': 613, 'commits': 35}
 
 
-def test_table_line():
-  """Test table line drawing"""
-  assert (_gitfame.tr_hline([3, 4, 2], hl='/', x='#') == '#///#////#//#')
-
-
 def test_tabulate():
   """Test builtin tabulate"""
-  assert (_gitfame.tabulate(auth_stats, stats_tot) ==
-          dedent("""\
+  assert (_gitfame.tabulate(auth_stats, stats_tot) == dedent("""\
     Total commits: 35
     Total files: 14
     Total loc: 613
-    +----------------------+-----+------+------+----------------+
-    | Author               | loc | coms | fils |  distribution  |
-    +======================+=====+======+======+================+
-    | Casper da Costa-Luis | 538 |   35 |   10 | 87.8/ 100/71.4 |
-    | Not Committed Yet    |  75 |    0 |    4 | 12.2/ 0.0/28.6 |
-    +----------------------+-----+------+------+----------------+"""))
+    | Author               |   loc |   coms |   fils |  distribution   |
+    |:---------------------|------:|-------:|-------:|:----------------|
+    | Casper da Costa-Luis |   538 |     35 |     10 | 87.8/ 100/71.4  |
+    | Not Committed Yet    |    75 |      0 |      4 | 12.2/ 0.0/28.6  |"""))
+
   sys.stderr.write("\rTest builtin tabulate ... ")  # `tqdm` may clear info
 
 
+def test_tabulate_cost():
+  """Test cost estimates"""
+  assert (_gitfame.tabulate(
+      auth_stats, stats_tot, cost="hours,COCOMO") == dedent("""\
+    Total commits: 35
+    Total files: 14
+    Total hours: 5.5
+    Total loc: 613
+    Total months: 1.9
+    | Author               |   hrs |   mths |   loc |   coms |   fils \
+|  distribution   |
+    |:---------------------|------:|-------:|------:|-------:|-------:\
+|:----------------|
+    | Casper da Costa-Luis |     4 |      2 |   538 |     35 |     10 \
+| 87.8/ 100/71.4  |
+    | Not Committed Yet    |     2 |      0 |    75 |      0 |      4 \
+| 12.2/ 0.0/28.6  |"""))
+
+
 def test_tabulate_yaml():
   """Test YAML tabulate"""
-  try:
-    assert (_gitfame.tabulate(auth_stats, stats_tot, backend='yaml') ==
-            dedent("""\
+  res = [
+      dedent("""\
+      columns:
+      - Author
+      - loc
+      - coms
+      - fils
+      - '%loc'
+      - '%coms'
+      - '%fils'
+      data:
+      - - Casper da Costa-Luis
+        - 538
+        - 35
+        - 10
+        - 87.8
+        - 100.0
+        - 71.4
+      - - Not Committed Yet
+        - 75
+        - 0
+        - 4
+        - 12.2
+        - 0.0
+        - 28.6
+      total:
+        commits: 35
+        files: 14
+        loc: 613"""),
+      # pyyaml<5
+      dedent("""\
       columns: [Author, loc, coms, fils, '%loc', '%coms', '%fils']
       data:
       - [Casper da Costa-Luis, 538, 35, 10, 87.8, 100.0, 71.4]
       - [Not Committed Yet, 75, 0, 4, 12.2, 0.0, 28.6]
-      total: {commits: 35, files: 14, loc: 613}"""))
+      total: {commits: 35, files: 14, loc: 613}""")]
+  try:
+    assert (_gitfame.tabulate(auth_stats, stats_tot, backend='yaml') in res)
   except ImportError:
     raise SkipTest
 
@@ -67,8 +116,7 @@ def test_tabulate_json():
   """Test JSON tabulate"""
   from json import loads
   res = loads(_gitfame.tabulate(auth_stats, stats_tot, backend='json'))
-  assert (res ==
-          loads(dedent("""\
+  assert (res == loads(dedent("""\
     {"total": {"files": 14, "loc": 613, "commits": 35},
     "data": [["Casper da Costa-Luis", 538, 35, 10, 87.8, 100.0, 71.4],
     ["Not Committed Yet", 75, 0, 4, 12.2, 0.0, 28.6]],
@@ -86,18 +134,15 @@ def test_tabulate_csv():
 def test_tabulate_tabulate():
   """Test external tabulate"""
   try:
-    assert (_gitfame.tabulate(auth_stats, stats_tot, backend='tabulate') ==
-            dedent("""\
+    assert (_gitfame.tabulate(
+        auth_stats, stats_tot, backend='simple') == dedent("""\
       Total commits: 35
       Total files: 14
       Total loc: 613
-      +----------------------+-------+--------+--------+-----------------+
-      | Author               |   loc |   coms |   fils |  distribution   |
-      +======================+=======+========+========+=================+
-      | Casper da Costa-Luis |   538 |     35 |     10 | 87.8/ 100/71.4  |
-      +----------------------+-------+--------+--------+-----------------+
-      | Not Committed Yet    |    75 |      0 |      4 | 12.2/ 0.0/28.6  |
-      +----------------------+-------+--------+--------+-----------------+"""))
+      Author                  loc    coms    fils   distribution
+      --------------------  -----  ------  ------  ---------------
+      Casper da Costa-Luis    538      35      10  87.8/ 100/71.4
+      Not Committed Yet        75       0       4  12.2/ 0.0/28.6"""))
   except ImportError:
     raise SkipTest
 
@@ -119,12 +164,12 @@ def test_main():
   import subprocess
   from os.path import dirname as dn
 
-  res = subprocess.Popen((sys.executable, '-c', "import gitfame; import sys;" +
-                          ' sys.argv = ["", "--silent-progress", "' +
-                          dn(dn(dn(__file__))) +
-                          '"]; gitfame.main()'),
-                         stdout=subprocess.PIPE,
-                         stderr=subprocess.STDOUT).communicate()[0]
+  res = subprocess.Popen(
+      (sys.executable, '-c', 'import gitfame; import sys;\
+       sys.argv = ["", "--silent-progress", "%s"];\
+       gitfame.main()' % dn(dn(dn(__file__)))),
+      stdout=subprocess.PIPE,
+      stderr=subprocess.STDOUT).communicate()[0]
 
   # actual test:
 
@@ -140,21 +185,23 @@ def test_main():
   # import gitfame.__main__  # NOQA
   main(['--silent-progress'])
 
+  sys.stdout.seek(0)
   try:
     main(['--bad', 'arg'])
   except SystemExit:
-    if """usage: gitfame [-h] [""" not in sys.stdout.getvalue():
-      raise
+    res = ' '.join(sys.stdout.getvalue().strip().split()[:2])
+    if res != "usage: gitfame":
+      raise ValueError(sys.stdout.getvalue())
+      raise ValueError(res)
   else:
     raise ValueError("Expected --bad arg to fail")
 
   sys.stdout.seek(0)
-  # import logging
-  # logging.basicConfig(level=logging.INFO, stream=sys.stdout)
-  main(['-s', '--sort', 'badSortArg'])
-  # if "--sort argument (badSortArg) unrecognised" \
-  #       not in sys.stdout.getvalue():
-  #   raise ValueError("Expected --sort argument (badSortArg) unrecognised")
+  try:
+    main(['-s', '--sort', 'badSortArg'])
+  except KeyError as e:
+    if "badSortArg" not in str(e):
+      raise ValueError("Expected `--sort=badSortArg` to fail")
 
   for params in [
       ['--sort', 'commits'],
